@@ -1,8 +1,6 @@
 #!/usr/bin/env python
 
-"""
-Command-line interface for Doorstop.
-"""
+"""Command-line interface for Doorstop."""
 
 import os
 import sys
@@ -10,51 +8,33 @@ import argparse
 import logging
 
 from doorstop import CLI, VERSION
-from doorstop.gui.main import run as gui
+from doorstop.gui.main import _run as gui
 from doorstop.core.tree import build
 from doorstop.core import report
 from doorstop import common
-from doorstop.common import DoorstopError
+from doorstop.common import DoorstopError, HelpFormatter, WarningFormatter
 from doorstop import settings
 
 
-# TODO: use the classes from doorstop.common
-class _HelpFormatter(argparse.HelpFormatter):
-    """Command-line help text formatter with wider help text."""
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, max_help_position=32, **kwargs)
-
-
-class _WarningFormatter(logging.Formatter, object):
-    """Logging formatter that always displays a verbose logging
-    format for logging level WARNING or higher."""
-
-    def __init__(self, default_format, verbose_format, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.default_format = default_format
-        self.verbose_format = verbose_format
-
-    def format(self, record):
-        """Python 3 hack to change the formatting style dynamically."""
-        if record.levelno > logging.INFO:
-            self._style._fmt = self.verbose_format  # pylint: disable=W0212
-        else:
-            self._style._fmt = self.default_format  # pylint: disable=W0212
-        return super().format(record)
-
-
 def main(args=None):  # pylint: disable=R0915
-    """Process command-line arguments and run the program.
-    """
+    """Process command-line arguments and run the program."""
     # Shared options
     debug = argparse.ArgumentParser(add_help=False)
+    debug.add_argument('-j', '--project', metavar='PATH',
+                       help="path to the root of the project")
     debug.add_argument('-V', '--version', action='version', version=VERSION)
     debug.add_argument('-v', '--verbose', action='count', default=0,
                        help="enable verbose logging")
-    shared = {'formatter_class': _HelpFormatter, 'parents': [debug]}
+    shared = {'formatter_class': HelpFormatter, 'parents': [debug]}
 
     # Main parser
     parser = argparse.ArgumentParser(prog=CLI, description=__doc__, **shared)
+    parser.add_argument('-F', '--no-reformat', action='store_true',
+                        help="do not reformat item files during validation")
+    parser.add_argument('-R', '--no-ref-check', action='store_true',
+                        help="do not validate external file references")
+    parser.add_argument('-L', '--no-rlinks-check', action='store_true',
+                        help="do not validate reverse links")
     parser.add_argument('-g', '--gui', action='store_true',
                         help="launch the graphical user interface")
     subs = parser.add_subparsers(help="", dest='command', metavar="<command>")
@@ -64,27 +44,27 @@ def main(args=None):  # pylint: disable=R0915
                           help="create a new document directory",
                           **shared)
     sub.add_argument('prefix', help="document prefix for new item IDs")
-    sub.add_argument('root', help="path to a directory for document items")
+    sub.add_argument('root', help="path to a directory for item files")
     sub.add_argument('-p', '--parent', help="prefix for parent item IDS")
     sub.add_argument('-d', '--digits', help="number of digits in item IDs")
 
     # Add subparser
     sub = subs.add_parser('add',
-                          help="add a new item to a document directory",
+                          help="create an item file in a document directory",
                           **shared)
     sub.add_argument('prefix',
                      help="document prefix for the new item")
 
     # Remove subparser
     sub = subs.add_parser('remove',
-                          help="remove an item from a document directory",
+                          help="remove an item file from a document directory",
                           **shared)
     sub.add_argument('id', metavar='ID',
-                     help="item ID to remove from a document")
+                     help="item ID to remove from its document")
 
     # Link subparser
     sub = subs.add_parser('link',
-                          help="add a new link between two document items",
+                          help="add a new link between two items",
                           **shared)
     sub.add_argument('child',
                      help="child item ID to link to the parent")
@@ -93,7 +73,7 @@ def main(args=None):  # pylint: disable=R0915
 
     # Unlink subparser
     sub = subs.add_parser('unlink',
-                          help="remove a link between two document items",
+                          help="remove a link between two items",
                           **shared)
     sub.add_argument('child',
                      help="child item ID to unlink from parent")
@@ -108,28 +88,19 @@ def main(args=None):  # pylint: disable=R0915
     sub.add_argument('-t', '--tool', metavar='PROGRAM',
                      help="text editor to open the document item")
 
-    # Import subparser
-    sub = subs.add_parser('import',
-                          help="import document items from another format",
-                          **shared)
-    sub.add_argument('input', help="file to import")
-
-    # Export subparser
-    sub = subs.add_parser('export',
-                          help="export document items to another format",
-                          **shared)
-    sub.add_argument('output', help="file to export")
-
     # Publish subparser
     sub = subs.add_parser('publish',
                           help="publish a document as text or another format",
                           **shared)
-    sub.add_argument('prefix', help="prefix of document to publish")
-    sub.add_argument('path', nargs='?', help="path of published file")
+    sub.add_argument('prefix', help="prefix of document to publish or 'all'")
+    sub.add_argument('path', nargs='?',
+                     help="path to published file or directory for 'all'")
+    sub.add_argument('-t', '--text', action='store_true',
+                     help="output text (default when no path)")
     sub.add_argument('-m', '--markdown', action='store_true',
-                     help="output Markdown instead of raw text")
+                     help="output Markdown")
     sub.add_argument('-H', '--html', action='store_true',
-                     help="output HTML converted from Markdown")
+                     help="output HTML (default for 'all')")
     sub.add_argument('-w', '--width', type=int,
                      help="limit line width on text output")
 
@@ -163,13 +134,19 @@ def main(args=None):  # pylint: disable=R0915
 
 def _configure_logging(verbosity=0):
     """Configure logging using the provided verbosity level (0+)."""
+    assert common.STR_VERBOSITY == 3
+    assert common.MAX_VERBOSITY == 4
 
     # Configure the logging level and format
     if verbosity == 0:
         level = settings.DEFAULT_LOGGING_LEVEL
         default_format = settings.DEFAULT_LOGGING_FORMAT
         verbose_format = settings.VERBOSE_LOGGING_FORMAT
-    elif verbosity in (1, 2):  # level 2 adds verbose string formatting
+    elif verbosity == 1:
+        level = settings.VERBOSE_LOGGING_LEVEL
+        default_format = settings.DEFAULT_LOGGING_FORMAT
+        verbose_format = settings.VERBOSE_LOGGING_FORMAT
+    elif verbosity == 2:
         level = settings.VERBOSE_LOGGING_LEVEL
         default_format = verbose_format = settings.VERBOSE_LOGGING_FORMAT
     elif verbosity == 3:
@@ -178,27 +155,41 @@ def _configure_logging(verbosity=0):
     else:
         level = settings.VERBOSE2_LOGGING_LEVEL
         default_format = verbose_format = settings.VERBOSE2_LOGGING_FORMAT
-    common.VERBOSITY = verbosity
 
     # Set a custom formatter
     logging.basicConfig(level=level)
-    formatter = _WarningFormatter(default_format, verbose_format)
+    formatter = WarningFormatter(default_format, verbose_format)
     logging.root.handlers[0].setFormatter(formatter)
 
     # Warn about excessive verbosity
-    if common.VERBOSITY > 4:
-        logging.warn("maximum supported verbosity level is 4")
-        common.VERBOSITY = 4
+    if verbosity > common.MAX_VERBOSITY:
+        msg = "maximum verbosity level is {}".format(common.MAX_VERBOSITY)
+        logging.warn(msg)
+        common.VERBOSITY = common.MAX_VERBOSITY
+    else:
+        common.VERBOSITY = verbosity
 
 
 def _run(args, cwd, err):  # pylint: disable=W0613
     """Process arguments and run the `doorstop` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
+    # Configure validation settings
+    if args.no_reformat is not None:
+        settings.REFORMAT = not args.no_reformat
+    if args.no_ref_check is not None:
+        settings.CHECK_REF = not args.no_ref_check
+    if args.no_rlinks_check is not None:
+        settings.CHECK_RLINKS = not args.no_rlinks_check
+
+    # Validate the tree
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
+        tree.load()
         valid = tree.valid()
     except DoorstopError as error:
         logging.error(error)
@@ -211,12 +202,14 @@ def _run(args, cwd, err):  # pylint: disable=W0613
 
 def _run_new(args, cwd, _):
     """Process arguments and run the `doorstop new` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
         document = tree.new(args.root, args.prefix,
                             parent=args.parent, digits=args.digits)
     except DoorstopError as error:
@@ -229,12 +222,14 @@ def _run_new(args, cwd, _):
 
 def _run_add(args, cwd, _):
     """Process arguments and run the `doorstop add` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
         item = tree.add(args.prefix)
     except DoorstopError as error:
         logging.error(error)
@@ -246,12 +241,14 @@ def _run_add(args, cwd, _):
 
 def _run_remove(args, cwd, _):
     """Process arguments and run the `doorstop remove` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
         item = tree.remove(args.id)
     except DoorstopError as error:
         logging.error(error)
@@ -263,12 +260,14 @@ def _run_remove(args, cwd, _):
 
 def _run_link(args, cwd, _):
     """Process arguments and run the `doorstop link` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
         child, parent = tree.link(args.child, args.parent)
     except DoorstopError as error:
         logging.error(error)
@@ -281,12 +280,14 @@ def _run_link(args, cwd, _):
 
 def _run_unlink(args, cwd, _):
     """Process arguments and run the `doorstop unlink` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
         child, parent = tree.unlink(args.child, args.parent)
     except DoorstopError as error:
         logging.error(error)
@@ -299,12 +300,14 @@ def _run_unlink(args, cwd, _):
 
 def _run_edit(args, cwd, _):
     """Process arguments and run the `doorstop edit` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
     try:
-        tree = build(cwd)
+        tree = build(cwd, root=args.project)
         item = tree.edit(args.id, tool=args.tool, launch=True)
     except DoorstopError as error:
         logging.error(error)
@@ -314,58 +317,63 @@ def _run_edit(args, cwd, _):
         return True
 
 
-def _run_import(args, cwd, err):
-    """Process arguments and run the `doorstop import` subcommand.
-    @param args: Namespace of CLI arguments
-    @param cwd: current working directory
-    @param err: function to call for CLI errors
-    """
-    logging.warning((args, cwd, err))
-    raise NotImplementedError("'doorstop import' not implemented")
-
-
-def _run_export(args, cwd, err):
-    """Process arguments and run the `doorstop export` subcommand.
-    @param args: Namespace of CLI arguments
-    @param cwd: current working directory
-    @param err: function to call for CLI errors
-    """
-    logging.warning((args, cwd, err))
-    raise NotImplementedError("'doorstop export' not implemented")
-
-
-def _run_publish(args, cwd, _):
+def _run_publish(args, cwd, err):
     """Process arguments and run the `doorstop report` subcommand.
+
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
+
     """
+    publish_tree = args.prefix == 'all'
     try:
-        tree = build(cwd)
-        document = tree.find_document(args.prefix)
+        tree = build(cwd, root=args.project)
+        if publish_tree:
+            documents = [document for document in tree]
+        else:
+            documents = [tree.find_document(args.prefix)]
     except DoorstopError as error:
         logging.error(error)
         return False
     else:
 
+        # Set base arguments
         kwargs = {'ignored': tree.vcs.ignored}
         if args.width:
             kwargs['width'] = args.width
 
+        # Set file extension
         if args.path:
-            ext = os.path.splitext(args.path)[-1]
-        if args.markdown:
+            if publish_tree:
+                ext = '.html'
+            else:
+                ext = os.path.splitext(args.path)[-1]
+        if args.text:
+            ext = '.txt'
+        elif args.markdown:
             ext = '.md'
         elif args.html:
             ext = '.html'
         elif not args.path:
             ext = '.txt'
 
+        # Publish documents
         if args.path:
-            print("publishing {} to {}...".format(document, args.path))
-            report.publish(document, args.path, ext, **kwargs)
+            if publish_tree:
+                print("publishing tree to {}...".format(args.path))
+                for document in documents:
+                    path = os.path.join(args.path, document.prefix + ext)
+                    print("publishing {} to {}...".format(document, path))
+                    report.publish(document, path, ext, **kwargs)
+                if ext == '.html':
+                    report.index(args.path)
+            else:
+                print("publishing {} to {}...".format(documents[0], args.path))
+                report.publish(documents[0], args.path, ext, **kwargs)
         else:
-            for line in report.iter_lines(document, ext, **kwargs):
+            if publish_tree:
+                err("only single documents can be displayed")
+            for line in report.lines(documents[0], ext, **kwargs):
                 print(line)
 
         return True
