@@ -1,15 +1,16 @@
-"""
-Classes and functions for objects whose attributes save to a file.
-"""
+"""Classes and functions for objects whose attributes save to a file."""
 
 import os
+import re
 import abc
 import functools
+import textwrap
 import logging
 
 import yaml
 
 from doorstop.common import DoorstopError
+from doorstop import settings
 
 
 def auto_load(func):
@@ -31,15 +32,38 @@ def auto_save(func):
         if self.auto:
             self.save()
         return result
+    return wrapped
 
+
+def clear_document_cache(func):
+    """Decorator for methods that should clear the document cache."""
+    @functools.wraps(func)
+    def wrapped(self, *args, **kwargs):
+        """Wrapped method to clear document cache after execution."""
+        result = func(self, *args, **kwargs)
+        self._document_cache.clear()  # pylint: disable=W0212
+        return result
+    return wrapped
+
+
+def clear_item_cache(func):
+    """Decorator for methods that should clear the item cache."""
+    @functools.wraps(func)
+    def wrapped(self, *args, **kwargs):
+        """Wrapped method to clear item cache after execution."""
+        result = func(self, *args, **kwargs)
+        self._item_cache.clear()  # pylint: disable=W0212
+        return result
     return wrapped
 
 
 class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
+
     """Abstract Base Class for objects whose attributes save to a file.
 
     For properties that are saved to a file, decorate their getters
     with @auto_load and their setters with @auto_save.
+
     """
 
     auto = True  # set to False to delay automatic save until explicit save
@@ -57,6 +81,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
         @param name: humanized name for this file
 
         @raise DoorstopError: if the file already exists
+
         """
         if os.path.exists(path):
             raise DoorstopError("{} already exists: {}".format(name, path))
@@ -83,6 +108,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
         @param path: path to a text file
 
         @return: contexts of text file
+
         """
         if not self._exists:
             raise DoorstopError("cannot read from deleted: {}".format(self))
@@ -97,6 +123,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
         @param path: path to the file (for displaying errors)
 
         @return: dictionary of YAML data
+
         """
         # Load the YAML data
         try:
@@ -124,11 +151,12 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
 
         @param text: text to write to a file
         @param path: path to the file
+
         """
         if not self._exists:
             raise DoorstopError("cannot save to deleted: {}".format(self))
-        with open(path, 'wb') as outfile:
-            outfile.write(bytes(text, 'UTF-8'))
+        with open(path, 'w') as outfile:
+            outfile.write(text)
 
     @staticmethod
     def _dump(data):
@@ -137,6 +165,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
         @param data: dictionary of YAML data
 
         @return: text to write to a file
+
         """
         return yaml.dump(data, default_flow_style=False)
 
@@ -160,6 +189,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
         @param default: value to return for missing attributes
 
         @return: value of extended attribute
+
         """
         if hasattr(self, name):
             cname = self.__class__.__name__
@@ -176,6 +206,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
 
         @param name: name of extended attribute
         @param value: value to set
+
         """
         if hasattr(self, name):
             cname = self.__class__.__name__
@@ -200,6 +231,7 @@ class BaseFileObject(object, metaclass=abc.ABCMeta):  # pylint:disable=R0921
 
 
 class Literal(str):  # pylint: disable=R0904
+
     """Custom type for text which should be dumped in the literal style."""
 
     @staticmethod
@@ -208,16 +240,99 @@ class Literal(str):  # pylint: disable=R0904
         return dumper.represent_scalar('tag:yaml.org,2002:str', data,
                                        style='|' if data else '')
 
-
-# TODO: use this class?
-class Folded(str):  # pragma: no cover, pylint: disable=R0904
-    """Custom type for text which should be dumped in the folded style."""
-
-    @staticmethod
-    def representer(dumper, data):
-        """Return a custom dumper that formats str in the folded style."""
-        return dumper.represent_scalar('tag:yaml.org,2002:str', data,
-                                       style='>' if data else '')
-
 yaml.add_representer(Literal, Literal.representer)
-yaml.add_representer(Folded, Folded.representer)
+
+# Based on: http://en.wikipedia.org/wiki/Sentence_boundary_disambiguation
+RE_SENTENCE_BOUNDARIES = re.compile(r"""
+
+(            # one of the following:
+
+  (?<=[a-z)][.?!])      # lowercase letter + punctuation
+  |
+  (?<=[a-z0-9][.?!]\")  # lowercase letter/number + punctuation + quote
+
+)
+
+(\s)          # any whitespace
+
+(?=\"?[A-Z])  # optional quote + an upppercase letter
+
+""", re.VERBOSE)
+
+
+def sbd(text, end='\n'):
+    r"""Replace sentence boundaries with newlines and append a newline.
+
+    @param text: string to line break at sentences
+    @param end: appended to the end of the update text
+
+    >>> sbd("Hello, world!", end='')
+    'Hello, world!'
+
+    >>> sbd("Hello, world! How are you? I'm fine. Good.")
+    "Hello, world!\nHow are you?\nI'm fine.\nGood.\n"
+
+    """
+    stripped = text.strip()
+    if stripped:
+        return RE_SENTENCE_BOUNDARIES.sub('\n', stripped) + end
+    else:
+        return ''
+
+
+def wrap(text, width=settings.MAX_LINE_LENTH):
+    r"""Wrap lines of text to the maximum line length.
+
+    >>> wrap("Hello, world!", 9)
+    'Hello,\nworld!'
+
+    >>> wrap("How are you?\nI'm fine.\n", 14)
+    "How are you?\nI'm fine.\n"
+
+    """
+    end = '\n' if '\n' in text else ''
+    lines = []
+    for line in text.splitlines():
+        # wrap longs lines of text compensating for the 2-space indent
+        lines.extend(textwrap.wrap(line, width=width - 2,
+                                   replace_whitespace=True))
+        if not line.strip():
+            lines.append('')
+    return '\n'.join(lines) + end
+
+
+RE_MARKDOWN_SPACES = re.compile(r"""
+
+([^\n ])  # any character but a newline or space
+
+(\ ?\n)     # optional space + single newline
+
+(?!      # none of the following:
+
+  (?:\s)       # whitespace
+  |
+  (?:[-+*]\s)  # unordered list separator + whitespace
+  |
+  (?:\d+\.\s)  # number + period + whitespace
+
+)
+
+([^\n])  # any character but a newline
+
+""", re.VERBOSE | re.IGNORECASE)
+
+
+def join(text):
+    r"""Convert single newlines (ignored by Markdown) to spaces.
+
+    >>> join("abc\n123")
+    'abc 123'
+
+    >>> join("abc\n\n123")
+    'abc\n\n123'
+
+    >>> join("abc \n123")
+    'abc 123'
+
+    """
+    return RE_MARKDOWN_SPACES.sub(r'\1 \3', text).strip()
